@@ -3,13 +3,13 @@ import os
 
 from subnet import ip_network, IPv4Network, IPv6Network
 
-from .base import WireGuardBase
+from .base import (
+    WireGuardBase,
+    KEEPALIVE_MINIMUM,
+)
 from ..utils import generate_key
 
 
-KEEPALIVE_MINIMUM = 5  # If you really need a keepalive value less than this, override the class
-
-
 class WireGuardPeer(WireGuardBase):
 
     endpoint = None
@@ -18,6 +18,8 @@ class WireGuardPeer(WireGuardBase):
     _keepalive = None
     _routable_ips = set()
 
+    server = None
+
     def __init__(self,
                  name,
                  subnet,
@@ -42,9 +44,10 @@ class WireGuardPeer(WireGuardBase):
             private_key=private_key,
             config_path=config_path,
             interface=interface,
-            server=server,
         )
 
+        self.server = server
+
         self.endpoint = endpoint
         self.server_pubkey = server_pubkey
         self.preshared_key = preshared_key
@@ -58,6 +61,38 @@ class WireGuardPeer(WireGuardBase):
             for ip in routable_ips:
                 self.add_routable_ip(ip)
 
+    @property
+    def private_key(self):
+        """
+        Returns the WireGuard private key associated with this client
+        """
+
+        if self._private_key is not None:
+            return self._private_key
+
+        self._private_key = generate_key()
+        if not self.server:
+            return self._private_key
+
+        count = 0
+        while count < MAXIMUM_KEY_RETRIES:
+            self._private_key = generate_key()
+            if self._private_key not in self.server.client_keys:
+                break
+            count += 1
+
+        if count >= MAXIMUM_KEY_RETRIES:
+            raise WireguardKeyGenerationError()
+
+        return self._private_key
+
+    @private_key.setter
+    def private_key(self, value):
+        if value is None:
+            raise ValueError('Private key cannot be empty')
+
+        self._private_key = value
+
     def add_routable_ip(self, ip):
         """
         Adds a routable IP to this config
@@ -79,6 +114,16 @@ class WireGuardPeer(WireGuardBase):
 
         return routable_ips
 
+    @routable_ips.setter
+    def routable_ips(self, value):
+        self._routable_ips = set()
+        if value is not None:
+            if not isinstance(value, list):
+                value = [value]
+
+            for ip in value:
+                self.add_routable_ip(ip)
+
     @property
     def preshared_key(self):
         """
@@ -110,15 +155,15 @@ class WireGuardPeer(WireGuardBase):
         Sets the keepalive value
         """
 
-        if not isinstance(value, int):
-            raise ValueError('Keepalive value must be an integer')
+        if value is not None:
+            if not isinstance(value, int):
+                raise ValueError('Keepalive value must be an integer')
 
-        if value < KEEPALIVE_MINIMUM:
-            value = KEEPALIVE_MINIMUM
+            if value < KEEPALIVE_MINIMUM:
+                value = KEEPALIVE_MINIMUM
 
         self._keepalive = value
 
-    @property
     def config(self):
         """
             Return the wireguard config file for this peer
@@ -151,7 +196,6 @@ PresharedKey = {self.preshared_key}
 
         return config
 
-    @property
     def serverside_config(self):
         """
         Return the server peer config for this client
